@@ -51,6 +51,13 @@ has expire_at => (
 	predicate => 'has_expiration',
 );
 
+has 'deliver_at' => (
+	is        => 'rw',
+	isa       => 'Num',
+	predicate => 'has_delay',
+	clearer   => 'clear_delay',
+);
+
 has claimant => (
 	is        => 'rw',
 	isa       => 'Maybe[Int]',
@@ -98,6 +105,26 @@ sub clone
 	return $self->meta->clone_object($self);
 }
 
+sub from_stomp_frame
+{
+	my ($class, $frame) = @_;
+	my $msg = $class->new(
+		id          => $frame->headers->{'message-id'},
+		destination => $frame->headers->{destination},
+		persistent  => $frame->headers->{persistent} eq 'true',
+		body        => $frame->body,
+	);
+	if (!$msg->persistent and my $after = $frame->headers->{'expire-after'})
+	{
+		$msg->expire_at(time + $after);
+	}
+	if (my $after = $frame->headers->{'deliver-after'})
+	{
+		$msg->deliver_at(time + $after);
+	}
+	return $msg;
+}
+
 sub create_stomp_frame
 {
 	my $self = shift;
