@@ -20,8 +20,6 @@ package POE::Component::MessageQueue::Statistics;
 use strict;
 use warnings;
 
-use Data::Dumper;
-
 sub new
 {
 	my $class = shift;
@@ -37,6 +35,7 @@ sub new
 			subscriptions => 0,
 			queues        => {},
 		},
+		store_info => {},
 		publishers => [],
 	}, $class;
 
@@ -47,7 +46,7 @@ sub register
 {
 	my ($self, $mq) = @_;
 	$mq->register_event( $_, $self ) 
-		for qw(store dispatch ack recv subscribe unsubscribe);
+		for qw(store dispatch remove recv subscribe unsubscribe);
 }
 
 sub add_publisher {
@@ -55,32 +54,20 @@ sub add_publisher {
   push(@{$self->{publishers}}, $pub);
 }
 
-my %METHODS = (
-	store       => 'notify_store',
-	'recv'      => 'notify_recv',
-	dispatch    => 'notify_dispatch',
-	ack         => 'notify_ack',
-	subscribe   => 'notify_subscribe',
-	unsubscribe => 'notify_unsubscribe',
-);
-
 sub get_queue
 {
 	my ($self, $name) = @_;
 
 	my $queue = $self->{statistics}{queues}{$name};
 
-	if ( not defined $queue )
+	unless ($queue)
 	{
-		$queue = $self->{statistics}{queues}{$name} = {
-			stored          => 0,
-			sent            => 0,
-			total_stored    => 0,
-			total_sent      => 0,
-			total_recvd     => 0,
-			avg_secs_stored => 0,
-			avg_size_recvd  => 0,
-		};
+		$queue = $self->{statistics}{queues}{$name} = {};
+		$queue->{$_} = 0 foreach qw(
+			sent          stored
+			total_stored  total_sent
+			total_recvd   avg_secs_stored  avg_size_recvd
+		);
 	}
 
 	return $queue;
@@ -112,22 +99,33 @@ sub shutdown
 sub notify
 {
 	my ($self, $event, $data) = @_;
-
-	my $method = $METHODS{ $event };
-	return unless $method;
-	$self->$method($data);
+	if(my $method = $self->can("notify_$event")) 
+	{
+		$method->($self, $data);
+	}
+	else
+	{
+		die "Tried to notify $event, which has no handler.";
+	}
 }
 
 sub notify_store
 {
-	my ($self, $data) = @_;
+	my ($self, $message) = @_;
+	my $qname = $message->destination;
 
-	# Global
-	my $h = $self->{statistics};
-	$h->{total_stored}++;
+	return unless $qname =~ m(/queue/(.*));
+	$qname = $1;
 
-	# Per-queue
-	my $stats = $self->get_queue($data->{queue}->{queue_name});
+	$self->{store_info}->{$message->id} = {
+		qname     => $qname,
+		timestamp => time(),
+	};
+
+	my $global = $self->{statistics};
+	$global->{total_stored}++;
+
+	my $stats = $self->get_queue($qname);
 	$stats->{stored}++;
 	$stats->{total_stored}++;
 	$stats->{last_stored} = scalar localtime();
@@ -142,8 +140,14 @@ sub reaverage {
 sub get_destination
 {
 	my ($self, $data) = @_;
-	return ($data->{queue} && $self->get_queue($data->{queue}->{queue_name})) ||
-	       ($data->{topic} && $self->get_topic($data->{topic}->{name}));
+	my $d = $data->{destination};
+	if ($d->name =~ m{/.*/(.*)})
+	{
+		return $d->isa('POE::Component::MessageQueue::Queue') ?
+		                                 $self->get_queue($1) :
+		                                 $self->get_topic($1) ;
+	}
+	return;
 }
 
 sub notify_recv
@@ -156,89 +160,36 @@ sub notify_recv
 	$stats->{avg_size_recvd} = reaverage(
 		$stats->{total_recvd},
 		$stats->{avg_size_recvd},
-		$data->{message}->{size},
+		$data->{message}->size,
 	);
 }
 
-sub message_handled
-{
-	my ($self, $data) = @_;
-
-	my $info = $data->{message} || $data->{message_info};
-
-	# Global
-	my $h = $self->{statistics};
-	$h->{total_sent}++;
-
-	# Per-queue
-	my $stats = $self->get_destination($data);
-
-	$stats->{sent}++;
-	$stats->{total_sent}++;
-	$stats->{last_sent} = scalar localtime();
-
-	# Topics don't count. :)
-	$stats->{stored}-- unless exists $data->{topic};
-
-	# We check if timestamp is set, because it might not be, in the specific
-	# case where the database was upgraded from pre-0.1.6.
-	# Topics don't get stored, so this doesn't make sense for them.
-	if ( $info->{timestamp} && not exists $data->{topic} )
-	{
-		# recalc the average
-		$stats->{avg_secs_stored} = reaverage(
-			$stats->{total_stored},
-			$stats->{avg_secs_stored},
-			(time() - $info->{timestamp}),
-		);
-	}
-}
-
 sub notify_dispatch
 {
 	my ($self, $data) = @_;
 
-	my $receiver = $data->{client};
-	if (exists $data->{topic})
-	{
-		$self->message_handled($data);
-		return;
-	}
+	my $global = $self->{statistics};
+	$global->{total_sent}++;
 
-	my $sub;
-	if ( ref($receiver) eq 'POE::Component::MessageQueue::Client' )
-	{
-		# automatically convert clients to subscribers!
-		$sub = $data->{queue}->get_subscription( $receiver );
-	}
-	else
-	{
-		$sub = $receiver;
-	}
-
-	if (($sub->{ack_type} || '') eq 'auto') {
-		$self->message_handled($data);
-	}
+	my $stats = $self->get_destination($data);
+	$stats->{total_sent}++;
+	$stats->{sent}++;
+	$stats->{last_sent} = scalar localtime();
 }
 
-sub notify_ack {
-	my ($self, $data) = @_;
+sub notify_remove {
+	my ($self, $id) = @_;
+	use YAML;
 
-	my $receiver = $data->{client};
-
-	my $sub;
-	if ( ref($receiver) eq 'POE::Component::MessageQueue::Client' )
+	if (my $store_info = delete $self->{store_info}->{$id})
 	{
-		# automatically convert clients to subscribers!
-		$sub = $data->{queue}->get_subscription( $receiver );
-	}
-	else
-	{
-		$sub = $receiver;
-	}
-
-	if (($sub->{ack_type} || '') eq 'client') {
-		$self->message_handled($data);
+		my $stats = $self->get_queue($store_info->{qname});
+		$stats->{stored}--;
+		$stats->{avg_secs_stored} = reaverage(
+			$stats->{total_stored},
+			$stats->{avg_secs_stored},
+			(time() - $store_info->{timestamp})
+		);
 	}
 }
 
